@@ -101,6 +101,7 @@ AVAILABILITY_INTERVAL = 86400 * 7  # 7 days
 NEVER_TIME = -AVAILABILITY_INTERVAL
 
 START_NOTIFY_DEBOUNCE = 1.5
+MAX_GSN = 65535
 
 SERVICE_INSTANCE_ID = "E604E95D-A759-4817-87D3-AA005083A0D1"
 SERVICE_INSTANCE_ID_UUID = UUID(SERVICE_INSTANCE_ID)
@@ -208,6 +209,7 @@ def disconnect_on_missing_services(func: WrapFuncType) -> WrapFuncType:
                 ex,
             )
             if self.client:
+                await self.client.clear_cache()
                 await self.client.disconnect()
             raise
 
@@ -266,6 +268,14 @@ class BlePairing(AbstractPairing):
 
         self._last_seen = time.monotonic() if description else NEVER_TIME
 
+        if not description and self.state_num:
+            self.description = HomeKitAdvertisement.from_cache(
+                address=self.address,
+                id=self.id,
+                config_num=self.config_num,
+                state_num=self.state_num,
+            )
+
         # Encryption
         self._derive = None
         self._session_id = None
@@ -305,6 +315,7 @@ class BlePairing(AbstractPairing):
         self._tried_to_connect_once = False
         self._restore_pending = False
         self._fetched_gsn_this_session = False
+        self._had_notify_this_session = False
 
     @property
     def address(self) -> str:
@@ -352,6 +363,18 @@ class BlePairing(AbstractPairing):
         """The transport used for the connection."""
         return Transport.BLE
 
+    def _update_state_num(self, state_num: int) -> None:
+        """Update the state number."""
+        self.description.state_num = state_num
+        self._update_cached_state_num(state_num)
+
+    def _update_cached_state_num(self, state_num: int) -> None:
+        """Update the cached state number which is restored between restarts."""
+        old_state_num = self._accessories_state.state_num
+        self._accessories_state.state_num = state_num
+        if old_state_num != state_num:
+            self._update_accessories_state_cache()
+
     def _async_ble_update(
         self, device: BLEDevice, ble_advertisement: AdvertisementData
     ) -> None:
@@ -377,6 +400,7 @@ class BlePairing(AbstractPairing):
             )
 
         super()._async_description_update(description)
+        self._update_cached_state_num(description.state_num)
 
     async def _async_request(
         self,
@@ -435,6 +459,7 @@ class BlePairing(AbstractPairing):
         self._broadcast_notifications = set()
         self._restore_pending = False
         self._fetched_gsn_this_session = False
+        self._had_notify_this_session = False
 
     async def _ensure_connected(self, attempts: int | None = None) -> bool | None:
         """Ensure that we are connected to the accessory.
@@ -499,17 +524,33 @@ class BlePairing(AbstractPairing):
                     await self._get_characteristics_while_connected(
                         [char], notify_listeners=True
                     )
-                    # After a char has changed we need to check if the
-                    # GSN has changed as well so we don't reconnect
-                    # to the accessory if we don't need to
-                    if self._fetched_gsn_this_session:
-                        # The spec says we should only do this once per session
-                        # after a characteristic has changed
-                        return
-                    protocol_param = await self._get_all_protocol_params()
-                    if protocol_param:
-                        self.description.state_num = protocol_param.state_number
-                        self._fetched_gsn_this_session = True
+                    # The GSN value is always the same per session.
+                    # It will never increment until the session is closed.
+                    #
+                    # If we haven't fetched the GSN this session, do it now.
+                    if not self._fetched_gsn_this_session and (
+                        protocol_param := await self._get_all_protocol_params()
+                    ):
+                        self._update_state_num(protocol_param.state_number)
+
+                    if not self._had_notify_this_session:
+                        self._had_notify_this_session = True
+                        # 7.4.1.8 Global State Number (GSN)
+                        # The GSN is only increased once per session
+                        # so we only want to increase it if we have
+                        # not already done so this session.
+                        #
+                        # The fetched GSN from _get_all_protocol_params
+                        # is NOT increased when we get a notify so we need
+                        # to do it here.
+                        #
+                        new_state_num = self.description.state_num + 1
+                        if new_state_num >= MAX_GSN:
+                            new_state_num = 1
+                            # GSN rolled over which invalidates the broadcast
+                            # encryption key. We need to re-fetch it.
+                            await self._async_set_broadcast_encryption_key()
+                        self._update_state_num(new_state_num)
 
         def _callback(id: int, data: bytes) -> None:
             logger.debug("%s: Received event for iid=%s: %s", self.name, iid, data)
@@ -528,11 +569,30 @@ class BlePairing(AbstractPairing):
 
     async def _async_pair_verify(self) -> None:
         async with self._ble_request_lock:
+            # There is only one pair verify characteristic
+            # per device so we can just use the first one
+            # and avoid having to request the iid which speeds
+            # up the pairing resume process
+            try:
+                pair_verify_char = (
+                    self.accessories.aid(BLE_AID)
+                    .services.first(service_type=ServicesTypes.PAIRING)
+                    .characteristics.first(
+                        char_types=[CharacteristicsTypes.PAIR_VERIFY]
+                    )
+                )
+            except StopIteration:
+                # If we don't have a pair verify characteristic yet
+                # it means we haven't fetched the services yet
+                pair_verify_char = None
+
             session_id, derive = await drive_pairing_state_machine(
                 self.client,
                 CharacteristicsTypes.PAIR_VERIFY,
                 get_session_keys(self.pairing_data, self._session_id, self._derive),
+                pair_verify_char.iid if pair_verify_char else None,
             )
+
             self._encryption_key = EncryptionKey(
                 derive(b"Control-Salt", b"Control-Write-Encryption-Key")
             )
@@ -580,7 +640,7 @@ class BlePairing(AbstractPairing):
                 return
 
             if protocol_param:
-                self.description.state_num = protocol_param.state_number
+                self._update_state_num(protocol_param.state_number)
 
     @operation_lock
     @retry_bluetooth_connection_error()
@@ -909,7 +969,7 @@ class BlePairing(AbstractPairing):
         """
         if not self.accessories and self.description:
             self._accessories_state = AccessoriesState(
-                Accessories(), -1, self.broadcast_key
+                Accessories(), -1, self.broadcast_key, self.state_num
             )
             return self.description.name
         return await super().get_primary_name()
@@ -989,6 +1049,7 @@ class BlePairing(AbstractPairing):
             protocol_params.state_number,
             protocol_params.config_number,
         )
+        self._fetched_gsn_this_session = True
         return protocol_params
 
     async def async_populate_accessories_state(
@@ -1161,7 +1222,7 @@ class BlePairing(AbstractPairing):
         # the state number again to make sure we are in sync
         protocol_param = await self._get_all_protocol_params()
         if protocol_param:
-            self.description.state_num = protocol_param.state_number
+            self._update_state_num(protocol_param.state_number)
         self._async_schedule_start_notify_subscriptions()
 
     @operation_lock
