@@ -250,8 +250,6 @@ class CoAPHomeKitConnection:
     async def reconnect_soon(self):
         if not self.enc_ctx:
             return
-        description = self.owner.description
-        self.address = f"[{description.address}]:{description.port}"
         await self.enc_ctx.coap_ctx.shutdown()
         self.enc_ctx = None
         # XXX can't .connect here w/o pairing_data
